#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008-2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import ldap
import csv
import os
import listener
import time
import ldap.modlist as modlist

uidblacklist=[ 'join-backup', 'join-slave', 'Administrator', 'spam', 'resourceadmin-']
oxAccess='premium'

oxbindirectory='/opt/open-xchange/sbin'
ldapserver = listener.configRegistry['ldap/server/name']
ldapbase = listener.configRegistry['ldap/base']
defaultdomain = listener.configRegistry['domainname']

try:
	admpwfilename=('/etc/ldap.secret')
	ldapadmpass=open(admpwfilename).read()
	if ldapadmpass[-1] == '\n':
		ldapadmpass=ldapadmpass[0:-1]

	username = ('cn=admin,%s' % ldapbase)

	l = ldap.open(ldapserver)
	l.protocol_version = ldap.VERSION3
	l.simple_bind(username, ldapadmpass)

except ldap.LDAPError, e:
	print e
	# handle error however you like

searchScope = ldap.SCOPE_SUBTREE

try:
	## migrate user attributes:
	retrieveAttributes = ['*']
	searchFilter = '(&(objectClass=oxUserObject)(!(objectClass=univentionUserTemplate)))'
	ldifaccessres = modlist.modifyModlist({'oxAccessResources':'OK'},{'oxAccessResources':''})
	ldap_result_id = l.search(ldapbase, searchScope, searchFilter, retrieveAttributes)
	result_set = []
	while 1:
		result_type, result_data = l.result(ldap_result_id, 0)
		if (result_data == []):
			break
		else:
			if result_type == ldap.RES_SEARCH_ENTRY:
				result_set.append(result_data)
	for result in result_set:
		userobject=result[0][1]
		userdn=result[0][0]
		uid = userobject['uid'][0]
#		if uid not in map(lambda x: x[0:len(x)], uidblacklist):
		if uid not in uidblacklist:
			if userobject.get('oxDisplayName'):
				oxDisplayName=userobject['oxDisplayName'][0]
			else:
				oxDisplayName=userobject['displayName'][0]
				try:
					l.modify_s(userdn, [ (ldap.MOD_ADD, 'oxDisplayName', oxDisplayName) ])
				except Exception, e:
					print "ldapmigrate error %s" %e

			if userobject.get('givenName'):
				givenName=userobject['givenName'][0]
			else:
				givenName='.'
				try:
					l.modify_s(userdn, [ (ldap.MOD_ADD, 'givenName', givenName) ])
				except Exception, e:
					print "ldapmigrate error %s" %e

			if userobject.get('telephoneNumber'):
				oxTelephoneBusiness1=userobject['telephoneNumber'][0]
				try:
					l.modify_s(userdn, [ (ldap.MOD_ADD, 'oxTelephoneBusiness1', oxTelephoneBusiness1) ])
				except Exception,e:
					print "ldapmigrate error %s" %e

				try:
					l.modify_s(userdn, [ (ldap.MOD_DELETE, 'telephoneNumber', None) ])
				except Exception,e:
					print "ldapmigrate error %s" %e

				if len(userobject['telephoneNumber']) > 1:
					oxTelephoneBusiness2=userobject['telephoneNumber'][1]
					try:
						l.modify_s(userdn, [ (ldap.MOD_ADD, 'oxTelephoneBusiness2', oxTelephoneBusiness2) ])
					except Exception,e:
						print "ldapmigrate error %s" %e

			if userobject.get('homePhone'):
				oxTelephoneHome1=userobject['homePhone'][0]
				try:
					l.modify_s(userdn, [ (ldap.MOD_ADD, 'oxTelephoneHome1', oxTelephoneHome1) ])
				except Exception,e:
					print "ldapmigrate error %s" %e

				if len(userobject['homePhone']) > 1:
					oxTelephoneHome2=userobject['homePhone'][1]
					try:
						l.modify_s(userdn, [ (ldap.MOD_ADD, 'oxTelephoneHome2', oxTelephoneHome2) ])
					except Exception,e:
						print "ldapmigrate error %s" %e

			print ('modifying object %s ' % oxDisplayName)
			if 'oxAccessResources' in userobject:
				try:
					l.modify_s(userdn,ldifaccessres)
				except Exception, e:
					print "ldapmigrate error %s" %e

			if 'oxAccess' in userobject:
				try:
					l.modify_s(userdn, [ (ldap.MOD_REPLACE, 'oxAccess', 'premium') ])
				except Exception, e:
					print "ldapmigrate error %s" %e
			else:
				try:
					l.modify_s(userdn, [ (ldap.MOD_ADD, 'oxAccess', 'premium') ])
				except Exception, e:
					print "ldapmigrate error %s" %e

		#if uid in uidblacklist
		else:
			if not 'givenName' in userobject:
				try:
					l.modify_s(userdn, [ (ldap.MOD_ADD, 'givenName', uid) ])
				except Exception, e:
					print "ldapmigrate error %s" %e

	## migrate resources:

	# find admin pw:
	retrieveAttributes = ['userPassword']
	searchFilter = '(uid=Administrator)'
	ldap_result_id = l.search(ldapbase, searchScope, searchFilter, retrieveAttributes)
	result_set = []
	while 1:
		result_type, result_data = l.result(ldap_result_id, 0)
		if (result_data == []):
			break
		else:
			if result_type == ldap.RES_SEARCH_ENTRY:
				result_set.append(result_data)
	adminPassword=result_set[0][0][1]['userPassword'][0]

	# find user template:
	retrieveAttributes = ['*']
	searchFilter = 'cn=open-xchange groupware account'
	ldap_result_id = l.search('cn=open-xchange groupware account, cn=templates,cn=univention,%s' % ldapbase, searchScope, searchFilter, retrieveAttributes)
	result_set = []
	while 1:
		result_type, result_data = l.result(ldap_result_id, 0)
		if (result_data == []):
			break
		else:
			if result_type == ldap.RES_SEARCH_ENTRY:
				result_set.append(result_data)
	if result_set[0][0][1].get('oxTimeZone'):
		oxTimeZone=result_set[0][0][1]['oxTimeZone'][0]
	else:
		oxTimeZone='Europe/Berlin'
	if result_set[0][0][1].get('oxLanguage'):
		oxLanguage=result_set[0][0][1]['oxLanguage'][0]
	else:
		oxLanguage='de_DE'

	if result_set[0][0][1].get('mailPrimaryAddress'):
		defaultdomain=result_set[0][0][1]['mailPrimaryAddress'][0].split('@')[1]

	#print 'timezone: %s, language: %s' % (oxTimeZone, oxLanguage)

	oxpwfilename=('/etc/ox-secrets/context10.secret')
	oxadmpass=open(oxpwfilename).read()
	if oxadmpass[-1] == '\n':
		oxadmpass=oxadmpass[0:-1]
	foundresourcescsvobject = False
	count=1
	while not foundresourcescsvobject and count <= 2:
		resourcescsvobject = csv.reader(os.popen('%s/listresource -A oxadmin -P "%s" -c 10 --csv' % (oxbindirectory, oxadmpass)))
		for csvline in resourcescsvobject:
			foundresourcescsvobject = True
		if not foundresourcescsvobject:
			time.sleep(5)
		count=count + 1
	resourcescsvobject = csv.reader(os.popen('%s/listresource -A oxadmin -P "%s" -c 10 --csv' % (oxbindirectory, oxadmpass)))
	count=1
	for csvline in resourcescsvobject:
		if csvline[0] != 'id':
			#print csvline
			rescn = csvline[1].replace('\n', ' ')
			resdisplayName=csvline[2]
			resmail=csvline[3]
			resdescription = csvline[5].replace('\n', ' ')
			retrieveAttributes = ['uidNumber']
			searchFilter = ('(|(mailPrimaryAddress=%s)(mailAlternativeAddress=%s))' % (resmail, resmail))
			ldap_result_id = l.search(ldapbase, searchScope, searchFilter, retrieveAttributes)
			result_set = []
			while 1:
				result_type, result_data = l.result(ldap_result_id, 0)
				if (result_data == []):
					break
				else:
					if result_type == ldap.RES_SEARCH_ENTRY:
						result_set.append(result_data)
			resUidNumber = ''
			for result in result_set:
				userobject=result[0][1]
				userdn=result[0][0]
				resUidNumber = userobject['uidNumber'][0]
			if resUidNumber == '':
				# create new user for resource:
				resusername=('resourceadmin-%s' % count)
				print 'creating user %s' % resusername
				os.popen('/usr/sbin/univention-directory-manager users/user create --binddn "%s" --bindpw "%s" --ignore_exists \
					--position "cn=users,%s" --set username="%s" --set firstname="%s" --set lastname="Admin" \
					--set firstname="Resource" --set password="dummy" --set overridePWLength=1 \
					--set mailPrimaryAddress="%s@%s" --set oxDisplayName="%s" --set oxTimeZone="%s" \
					--set oxLanguage="%s" > /dev/null 2>&1' \
					% ( username, ldapadmpass, ldapbase, resusername, resusername, resusername, defaultdomain, \
						resusername, oxTimeZone, oxLanguage ))
				# wait for user object to be created and get uidNumber:
				retrieveAttributes = ['uidNumber']
				searchFilter = ('(uid=%s)' % resusername)
				searchcount=1
				while resUidNumber == '' and searchcount <=1:
					ldap_result_id = l.search(ldapbase, searchScope, searchFilter, retrieveAttributes)
					result_set = []
					while 1:
						result_type, result_data = l.result(ldap_result_id, 0)
						if (result_data == []):
							break
						else:
							if result_type == ldap.RES_SEARCH_ENTRY:
								result_set.append(result_data)
					if result_set and result_set[0][0][1].get('uidNumber'):
						resUidNumber=result_set[0][0][1]['uidNumber'][0]
						resuserdn=result_set[0][0]
						#print 'found dN %s, uidNumber: %s' % (resuserdn, resUidNumber)
					else:
						#print '.'
						time.sleep(5)
						searchcount=searchcount + 1

				# change password to Administrators password:
				print 'changing password of user %s' % resusername
				try:
					l.modify_s('uid=%s,cn=users,%s' % (resusername, ldapbase), [ (ldap.MOD_REPLACE, 'userPassword', '%s' % adminPassword) ])
				except Exception, e:
					print "ldapmigrate error %s" %e

				count=count + 1
			print 'creating resource %s' % rescn
			os.popen ('/usr/sbin/univention-directory-manager oxresources/oxresources create --binddn "%s" --bindpw "%s" '\
				'--position "cn=oxresources,cn=open-xchange,%s" ' \
				'--set displayname="%s" --set oxenableresource=OK --set name="%s" ' \
				'--set resourceadmin="%s" > /dev/null 2>&1'\
				% ( username, ldapadmpass, ldapbase, resdisplayName, rescn, resUidNumber ))
			# change resource to force sync:
			print 'changing resource %s' % rescn
			os.popen ('/usr/sbin/univention-directory-manager oxresources/oxresources modify --binddn "%s" --bindpw "%s" '\
				'--dn "cn=%s,cn=oxresources,cn=open-xchange,%s" ' \
				'--set description="%s" > /dev/null 2>&1'
				% ( username, ldapadmpass, rescn, ldapbase, resdescription ))

	## delete fetchmail attributs:
	retrieveAttributes = ['*']
	searchFilter = 'objectClass=univentionFetchmail'
	ldap_result_id = l.search(ldapbase, searchScope, searchFilter, retrieveAttributes)
	result_set = []
	while 1:
		result_type, result_data = l.result(ldap_result_id, 0)
		if (result_data == []):
			break
		else:
			if result_type == ldap.RES_SEARCH_ENTRY:
				result_set.append(result_data)
	for result in result_set:
		ldifmodlist = [	( ldap.MOD_DELETE, 'objectClass', 'univentionFetchmail' ) ]
		userobject=result[0][1]
		objectdn=result[0][0]
		if 'univentionFetchmailAddress' in userobject:
			ldifmodlist.append((ldap.MOD_DELETE, 'univentionFetchmailAddress', None))
		if 'univentionFetchmailServer' in userobject:
			ldifmodlist.append((ldap.MOD_DELETE, 'univentionFetchmailServer', None))
		if 'univentionFetchmailProtocol' in userobject:
			ldifmodlist.append((ldap.MOD_DELETE, 'univentionFetchmailProtocol', None))
		if 'univentionFetchmailPasswd' in userobject:
			ldifmodlist.append((ldap.MOD_DELETE, 'univentionFetchmailPasswd', None))
		if 'univentionFetchmailKeepMailOnServer' in userobject:
			ldifmodlist.append((ldap.MOD_DELETE, 'univentionFetchmailKeepMailOnServer', None))
		try:
			l.modify_s(objectdn,ldifmodlist)
		except Exception, e:
			print "ldapmigrate error %s" %e

	## delete fetchmail attributs:
	retrieveAttributes = ['*']
	searchFilter = 'objectClass=univentionFetchmailConfig'
	ldap_result_id = l.search(ldapbase, searchScope, searchFilter, retrieveAttributes)
	result_set = []
	while 1:
		result_type, result_data = l.result(ldap_result_id, 0)
		if (result_data == []):
			break
		else:
			if result_type == ldap.RES_SEARCH_ENTRY:
				result_set.append(result_data)
	for result in result_set:
		ldifmodlist = [	( ldap.MOD_DELETE, 'objectClass', 'univentionFetchmailConfig' ) ]
		userobject=result[0][1]
		objectdn=result[0][0]
		if 'univentionFetchmailProtocolType' in userobject:
			ldifmodlist.append((ldap.MOD_DELETE, 'univentionFetchmailProtocolType', None))

		try:
			l.modify_s(objectdn,ldifmodlist)
		except Exception, e:
			print "ldapmigrate error %s" %e
except ldap.LDAPError, e:
	print e
