#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling domains
#
# Copyright (C) 2004-2015 Univention GmbH <http://www.univention.de/>
# and
#     iKu Systems & Services GmbH & Co. KG (http://www.iku-systems.de/)
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = ''  # workaround for PEP 366
import os
import univention.debug as ud
from univention.ox.listener_tools import ListenerQueue, AsRoot, get_credentials, ox_list_context, ox_call, OX_MASTER_ADMIN, OX_MASTER_SECRET

name = 'ox-context'
description = 'Update Open-Xchange context'
filter = '(objectClass=oxContext)'
attributes = []

need_restart = False

CSV_CONTEXT_COL_ID = 'id'
CSV_CONTEXT_COL_FILESTORE = 'filestore_id'


@ListenerQueue(name, stop_on_error=False)
def handler(dn, new, old, action=None):
	if not new:
		return

	admpass = get_credentials(OX_MASTER_SECRET)

	oxquota = '-1'
	if new.get('oxQuota'):
		oxquota = new['oxQuota'][0]
	if oxquota == '0':
		oxquota = '-1'
	ud.debug(ud.LISTENER, ud.INFO, 'ox-context: setting quota: %s' % oxquota)
	command = [
		'changecontext',
		'--adminuser=%s' % OX_MASTER_ADMIN,
		'--adminpass=%s' % admpass,
		'--contextid', '%s' % new['oxContextIDNum'][0],
		'--quota', '%s' % oxquota,
	]
	ox_call(command)

	# get filestore id for context
	filestoreId = False
	for row in ox_list_context(OX_MASTER_ADMIN, admpass):
		if row[CSV_CONTEXT_COL_ID] == new['oxContextIDNum'][0]:
			filestoreId = row[CSV_CONTEXT_COL_FILESTORE]
	if filestoreId:
		command = [
			'changefilestore',
			'--adminuser=%s' % OX_MASTER_ADMIN,
			'--adminpass=%s' % admpass,
			'--id', filestoreId,
			'--storesize', '%s' % oxquota
		]
		ox_call(command)

		global need_restart
		need_restart = True


def postrun():
	global need_restart
	if need_restart:
		with AsRoot():
			os.spawnv(os.P_WAIT, '/etc/init.d/open-xchange', ['open-xchange', 'restart'])
		need_restart = False

if __name__ == '__main__':
	handler.process_pending()
