#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling domains
#
# Copyright (C) 2004-2015 Univention GmbH <http://www.univention.de/>
# and
#     iKu Systems & Services GmbH & Co. KG (http://www.iku-systems.de/)
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = ''  # workaround for PEP 366
from univention.config_registry import ConfigRegistry
from univention.ox.listener_tools import ListenerQueue, get_credentials, ox_call, OX_MASTER_ADMIN, OX_MASTER_SECRET

name = 'ox-domain'
description = 'Update Open-Xchange domains'
filter = '(&(objectClass=oxMailDomainServerSettings)(cn=*))'
attributes = []


@ListenerQueue(name, stop_on_error=False)
def handler(dn, new, old, action=None):
	admpass = get_credentials(OX_MASTER_SECRET)

	ucr = ConfigRegistry()
	ucr.load()

	if old:
		remove_context(old, admpass, ucr)

	if new:
		add_context(new, admpass, ucr)


def remove_context(old, admpass, ucr):
	try:
		context = old['oxContextName'][0]
		cn = old['cn'][0]
	except LookupError:
		return
	command = [
		'changecontext',
		'--adminuser=%s' % OX_MASTER_ADMIN,
		'--adminpass=%s' % admpass,
		'--contextname', context,
		'--removemapping=%s' % (cn,),
	]
	ox_call(command)


def add_context(new, admpass, ucr):
	try:
		context = new['oxContextName'][0]
		cn = new['cn'][0]
	except LookupError:
		return
	command = [
		'changecontext',
		'--adminuser=%s' % OX_MASTER_ADMIN,
		'--adminpass=%s' % admpass,
		'--contextname', context,
		'--addmapping=%s' % (cn,),
	]
	ox_call(command)


if __name__ == '__main__':
	handler.process_pending()
