#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling groups
#
# Copyright (C) 2004-2015 Univention GmbH <http://www.univention.de/>
# and
#     iKu Systems & Services GmbH & Co. KG (http://www.iku-systems.de/)
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = ''  # workaround for PEP 366
import listener
import univention.debug as ud
import univention.uldap
from univention.ox.listener_tools import ListenerQueue, AsRoot, get_credentials, ListenerTools_RuntimeError, ox_list_user, ox_list_group, ox_call
from ldap import NO_SUCH_OBJECT

import os

name = 'ox-groups'
description = 'Group settings for Open-Xchange'
filter = '(objectClass=posixGroup)'
attributes = []

ldapserver = listener.configRegistry['ldap/server/name']
ldapbase = listener.configRegistry['ldap/base']
myhostname = listener.configRegistry['hostname']
mydomainname = listener.configRegistry['domainname']
myfqdn = '%s.%s' % (myhostname, mydomainname)
weblang = listener.configRegistry.get("locale/default", "").split(':')[0]

csv_user_row_uid = "Id"
csv_user_row_username = "Name"
csv_group_row_groupname = "name"
csv_group_row_members = "members"

def __is_groupType_universal(self, adGroupType):
	try:
		return int(adGroupType) & 0x8
	except ValueError:
		return False

def __is_groupType_global(self, adGroupType):
	try:
		return int(adGroupType) & 0x2
	except ValueError:
		return False

@ListenerQueue(name, stop_on_error=False)
def handler(dn, new, old, action=None):
	ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Running ox-group.py-handler')

	with AsRoot():
		try:
			ldapCon = univention.uldap.getMachineConnection(ldap_master=False)
		except IOError as ex:
			raise ListenerTools_RuntimeError(ex)

	# language
	os.environ["LANG"] = weblang

	mycontexts = []
	filter = "(&(objectClass=oxContext)(oxHomeServer=%s))" % myfqdn
	try:
		result_id = ldapCon.search(base="cn=open-xchange,%s" % ldapbase, filter=filter, attr=['oxContextIDNum', 'cn'])
	except NO_SUCH_OBJECT as ex:
		raise ListenerTools_RuntimeError(ex)
	if result_id:
		for contextnumelement in result_id:
			contextnum = contextnumelement[1]['oxContextIDNum'][0]
			contextname = contextnumelement[1]['cn'][0]
			ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Contextnum: %s' % contextnum)
			ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Contextname: %s' % contextname)
			mycontexts.append([contextnum, contextname])
	ud.debug(ud.LISTENER, ud.INFO, 'ox-group: My Contexts (id, name): %s' % mycontexts)
	groupname = ''
	if old.get('cn'):
		groupname = old['cn'][0]
	if new.get('cn'):
		groupname = new['cn'][0]

	if new.get('memberUid'):
		memberuids = new['memberUid']
	else:
		memberuids = []

	if new.get('description'):
		description = new['description'][0]
	else:
		description = groupname
	description = description[:128]  # OX length limit (Bug #35820)

	# ignore groups with name "users" (Bug #35821)
	if groupname.lower() == "users":
		return

	oldOxGroup = old.get('isOxGroup', [''])[0]
	newOxGroup = new.get("isOxGroup", [""])[0]

	if newOxGroup.lower() == "ok":
		gidnumber = new['gidNumber'][0]
		ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Group %s is OX group' % groupname)
		filter = '(&(objectClass=oxUserObject)(gidNumber=%s))' % gidnumber
		result_id = ldapCon.search(base=ldapbase, filter=filter, attr=['uid'])
		# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Primary users: %s' % result_id)
		if result_id:
			for userelement in result_id:
				# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Primary users: %s' % userelement[1].values()[0])
				memberuids.extend(userelement[1].values()[0])
		if not memberuids:
			memberuids = []
		if True:
			domainsearchstring = '(&(objectClass=oxUserObject)(mailPrimaryAddress=*)(|'
			for memberuid in memberuids:
				# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Processing uid %s' % memberuid)
				domainsearchstring = ('%s(uid=%s)' % (domainsearchstring, memberuid))
			domainsearchstring = '%s))' % domainsearchstring
			ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Domainsearchstring: %s, ldap_base: %s' % (domainsearchstring, ldapbase))
			usermailmap = []
			result_id = ldapCon.search(base=ldapbase, filter=domainsearchstring, attr=['mailPrimaryAddress', 'uid'])
			if result_id:
				for mailelement in result_id:
					ud.debug(ud.LISTENER, ud.INFO, 'ox-group: mailelement: %s' % mailelement[1]['mailPrimaryAddress'])
					ud.debug(ud.LISTENER, ud.INFO, 'ox-group: mailelement: %s' % mailelement[1].values()[1][0])
					try:
						domain = mailelement[1]['mailPrimaryAddress'][0].split('@')[1]
						uid = mailelement[1]['uid'][0]
						usermailmap.append([uid, domain])
						ud.debug(ud.LISTENER, ud.PROCESS, 'ox-group: user, domain: %s, %s' % (uid, domain))
					except:
						pass
			ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Group usermap: %s' % usermailmap)
			for context in mycontexts:
				# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Processing context num, name %s, %s' % (context [0], context[1]))
				admuser = 'oxadmin-context%s' % context[0]
				if admuser == 'oxadmin-context10':
					admuser = 'oxadmin'
				admpass = get_credentials('/etc/ox-secrets/context%s.secret' % context[0])
				# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: admpass: %s' % admpass)

				contextdomains = []
				filter = '(&(objectClass=univentionMailDomainname)(oxContextName=%s))' % context[1]
				result_id = ldapCon.search(base="cn=mail,%s" % ldapbase, filter=filter, attr=['cn'])
				if result_id:
					for domainelement in result_id:
						domain = domainelement[1].values()[0][0]
						# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Processing domain: %s' % domain)
						contextdomains.append(domain)
					# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Context Domains: %s' % contextdomains)
				# user dieses contexts user domain suchen:
				contextusers = []
				for uidmail in usermailmap:
					if uidmail[1] in contextdomains:
						if uidmail[0] in memberuids:  # FIXME: PMH: this will never be false, as usermailmap is built from memberuids through LDAP lookups
							contextusers.append(uidmail[0])
				# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Context Users: %s' % contextusers)

				contextuserids = []
				for row in ox_list_user(admuser, admpass, context[0]):
					# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Testing User: %s, %s' % (row[csv_user_row_username], row[csv_user_row_uid]))
					for contextuser in contextusers:
						# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Processing User: %s' % contextuser)
						if row[csv_user_row_username] == contextuser:
							# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Found User: %s, %s' % (row[csv_user_row_username], row[csv_user_row_uid]))
							contextuserids.append(row[csv_user_row_uid])
							break

				ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Context Users IDs: %s' % contextuserids)
				oldcontextuserids = []
				groupexists = False
				for oxgroupelement in ox_list_group(admuser, admpass, context[0], searchpattern=groupname):
					if oxgroupelement[csv_group_row_groupname] == groupname:
						# ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Processing OX-Group: %s' % oxgroupelement[1])
						oldcontextuseridstring = oxgroupelement[csv_group_row_members]
						oldcontextuserids = oldcontextuseridstring.split(',')
						groupexists = True
				ud.debug(ud.LISTENER, ud.INFO, 'ox-group: Old Group Member IDs: %s' % oldcontextuserids)
				# Mitglieder vergleichen

				addmembers = ''
				for contextuserid in contextuserids:
					if not (contextuserid in oldcontextuserids):
						addmembers = '%s,%s' % (contextuserid, addmembers)
				addmembers = addmembers[0:-1]
				ud.debug(ud.LISTENER, ud.INFO,
					'ox-group: addmembers: %s' % addmembers)

				removemembers = ''
				for contextuserid in oldcontextuserids:
					if not (contextuserid in contextuserids):
						removemembers = '%s,%s' % (contextuserid, removemembers)
				removemembers = removemembers[0:-1]
				ud.debug(ud.LISTENER, ud.INFO,
					'ox-group: removemembers: %s' % removemembers)

				command = []
				if groupexists:
					ud.debug(ud.LISTENER, ud.INFO,
						'ox-group: change group %s' % groupname)
					command = [
						"changegroup",
						"--displayname=%s" % description,
					]
					if removemembers:
						command.append("--removemembers=%s" % removemembers)
					if addmembers:
						command.append("--addmembers=%s" % addmembers)
				else:
					ud.debug(ud.LISTENER, ud.PROCESS,
						'ox-group: create group %s' % groupname)
					command = [
						"creategroup",
						"--displayname=%s" % description,
					]
					if addmembers:
						command.append("--addmembers=%s" % addmembers)
				command += [
					"--adminuser=%s" % admuser,
					"--adminpass=%s" % admpass,
					"--contextid=%s" % context[0],
					"--name=%s" % groupname,
				]
				ox_call(command)
				ud.debug(ud.LISTENER, ud.INFO, 'ox-group: done.')

	# isOxGroup has syntax OkOrNot
	if oldOxGroup.lower() == "ok" and (not new or newOxGroup.lower() == "not"):
		for context in mycontexts:
			ud.debug(ud.LISTENER, ud.PROCESS, 'ox-group: delete group %s' % groupname)
			admuser = 'oxadmin-context%s' % context[0]
			if admuser == 'oxadmin-context10':
				admuser = 'oxadmin'
			admpass = get_credentials('/etc/ox-secrets/context%s.secret' % context[0])
			deleteCommand = [
				"deletegroup",
				"--adminuser=%s" % admuser,
				"--adminpass=%s" % admpass,
				"--contextid=%s" % context[0],
				"--name=%s" % groupname,
			]
			ox_call(deleteCommand, "com.openexchange.admin.rmi.exceptions.NoSuchObjectException")


if __name__ == '__main__':
	handler.process_pending()
