#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling Open-Xchange resources
#
# Copyright (C) 2004-2015 Univention GmbH <http://www.univention.de/>
# and
#     iKu Systems & Services GmbH & Co. KG (http://www.iku-systems.de/)
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = ''  # workaround for PEP 366
import listener
import univention.debug as ud
from univention.ox.listener_tools import ListenerQueue, get_credentials, ox_call, debug

import os

name = 'ox-resource'
description = 'Resource settings for Open-Xchange'
filter = '(objectClass=oxResourceObject)'
attributes = []

weblang = listener.configRegistry.get("locale/default", "").split(':')[0]


@ListenerQueue(name, stop_on_error=False)
def handler(dn, new, old, action=None):
	os.environ["LANG"] = weblang

	if new:  # add | modify | move_to
		resourcename = new['cn'][0]

		email = new.get('mailPrimaryAddress', [''])[0]
		if not email:
			return

		if old:  # modify
			debug(ud.PROCESS, 'Modifying Open-Xchange resource %s' % (resourcename,))
			command = ["changeresource"]
		else:  # add | move_to
			debug(ud.PROCESS, 'Creating Open-Xchange resource %s' % (resourcename,))
			command = ["createresource"]

		command += [
			'--email=%s' % email,
			'--available=true',
		]
		if new.get('displayName'):
			command.append('--displayname=%s' % new['displayName'][0])
		if new.get('description'):
			command.append('--description=%s' % new['description'][0])
	elif old:  # delete | move_from
		resourcename = old['cn'][0]
		debug(ud.PROCESS, 'Deleting Open-Xchange resource %s' % (resourcename,))
		command = ["deleteresource"]
	else:
		return

	admuser, admpass = get_context_credentials(10)

	command += [
		"--adminuser=%s" % admuser,
		"--adminpass=%s" % admpass,
		"--contextid=10",
		"--name=%s" % resourcename,
	]

	ox_call(command)


def get_context_credentials(contextid):
	admuser = 'oxadmin-context%s' % (contextid,)
	if admuser == 'oxadmin-context10':
		admuser = 'oxadmin'
	admpass = get_credentials('/etc/ox-secrets/context%s.secret' % (contextid,))
	return admuser, admpass


if __name__ == '__main__':
	handler.process_pending()
