#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
#  OX for UCS
#  listener module: handling users
#
# Copyright (C) 2004-2015 Univention GmbH <http://www.univention.de/>
# and
#     iKu Systems & Services GmbH & Co. KG (http://www.iku-systems.de/)
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = ''  # workaround for PEP 366
import listener
import univention.debug as ud
import univention.uldap
from univention.ox.listener_tools import ListenerQueue, AsRoot, get_credentials, ListenerTools_RuntimeError, ox_list_user, ox_list_group, ox_call

import os
import subprocess

ldap2ox = {
	'oxDisplayName': 'displayname',
	'title': 'title',
	'givenName': 'givenname',
	'oxMiddleName': 'middle_name',
	'sn': 'surname',
	'oxSuffix': 'suffix',
	'oxNickName': 'nickname',
	'o': 'company',
	'oxDepartment': 'department',
	'oxPosition': 'position',
	'oxProfession': 'profession',
	'oxBranches': 'branches',
	'oxMobileBusiness': 'cellular_telephone1',
	'mobile': 'cellular_telephone2',
	'oxTelephoneCompany': 'telephone_company',
	'oxTelephoneOther': 'telephone_other',
	'oxTelephoneCar': 'telephone_car',
	'oxFaxBusiness': 'fax_business',
	'oxFaxHome': 'fax_home',
	'oxFaxOther': 'fax_other',
	'oxManagerName': 'manager_name',
	'oxTelephoneAssistant': 'telephone_assistant',
	'mailPrimaryAddress': 'email1',
	'oxEmail2': 'email2',
	'oxEmail3': 'email3',
	'oxInstantMessenger1': 'instant_messenger1',
	'oxInstantMessenger2': 'instant_messenger2',
	'oxTelephoneIp': 'telephone_ip',
	'pager': 'telephone_pager',
	'oxTelephoneTelex': 'telephone_telex',
	'oxTelephoneTtydd': 'telephone_ttytdd',
	'oxUrl': 'url',
	'oxTaxId': 'tax_id',
	'oxCommercialRegister': 'commercial_register',
	'oxSalesVolume': 'sales_volume',
	'oxMarialStatus': 'marital_status',
	'oxSpouseName': 'spouse_name',
	'street': 'street_business',
	'postalCode': 'postal_code_business',
	'l': 'city_business',
	'oxStateBusiness': 'state_business',
	'oxCountryBusiness': 'country_business',
	'oxStreetHome': 'street_home',
	'oxPostalCodeHome': 'postal_code_home',
	'oxCityHome': 'city_home',
	'oxStateHome': 'state_home',
	'oxCountryHome': 'country_home',
	'oxStreetOther': 'street_other',
	'oxPostalCodeOther': 'postal_code_other',
	'oxCityOther': 'city_other',
	'oxStateOther': 'state_other',
	'oxCountryOther': 'country_other',
	'oxUserfield01': 'userfield01',
	'oxUserfield02': 'userfield02',
	'oxUserfield03': 'userfield03',
	'oxUserfield04': 'userfield04',
	'oxUserfield05': 'userfield05',
	'oxUserfield06': 'userfield06',
	'oxUserfield07': 'userfield07',
	'oxUserfield08': 'userfield08',
	'oxUserfield09': 'userfield09',
	'oxUserfield10': 'userfield10',
	'oxUserfield11': 'userfield11',
	'oxUserfield12': 'userfield12',
	'oxUserfield13': 'userfield13',
	'oxUserfield14': 'userfield14',
	'oxUserfield15': 'userfield15',
	'oxUserfield16': 'userfield16',
	'oxUserfield17': 'userfield17',
	'oxUserfield18': 'userfield18',
	'oxUserfield19': 'userfield19',
	'oxUserfield20': 'userfield20',
	'employeeType': 'categories',
	'oxNote': 'note',
	}

ldap2oxmulti = {
	'telephoneNumber': 'telephone_business',
	'homePhone': 'telephone_home',
}

name = 'ox-user'
description = 'User settings for Open-Xchange'
filter = '(&(objectClass=inetOrgPerson)(objectClass=oxUserObject))'
attributes = ['uid', 'uidNumber', 'gidNumber', 'mailAlternativeAddress', 'roomNumber', 'employeeNumber',
	'univentionPolicyReference', 'isOxUser', 'oxBirthday', 'oxAnniversary', 'oxNumOfChildren',
	'oxAccess', 'oxAccessSyncMl', 'oxAccessResources', 'oxLanguage', 'oxTimeZone', 'oxAccessUSM']
attributes.extend(ldap2ox.keys())
attributes.extend(ldap2oxmulti.keys())
# use the modrdn listener extension
modrdn = "1"

ldapserver = listener.configRegistry['ldap/server/name']
ldapbase = listener.configRegistry['ldap/base']
myhostname = listener.configRegistry['hostname']
mydomainname = listener.configRegistry['domainname']
myfqdn = '%s.%s' % (myhostname, mydomainname)
weblang = listener.configRegistry.get("locale/default", "").split(':')[0]

csv_user_row_uid = "Id"
csv_user_row_username = "Name"
csv_user_row_default_sender_address = "DefaultSenderAddress"
csv_group_row_groupname = "name"
csv_group_row_members = "members"


@ListenerQueue(name, stop_on_error=False)
def handler(dn, new, old, command):
	# language
	os.environ["LANG"] = weblang

	oldIsOxUser = False
	if old:
		ud.debug(ud.LISTENER, ud.INFO, 'ox-user: Old present.')
		if old.get('isOxUser') and old['isOxUser'][0] == 'OK':
			oldIsOxUser = True

	newIsOxUser = False
	if new:
		ud.debug(ud.LISTENER, ud.INFO, 'ox-user: New present.')
		if new.get('isOxUser') and new['isOxUser'][0] == 'OK':
			newIsOxUser = True
	ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: New Object is OX-User: %s' % newIsOxUser)
	ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: Old Object is OX-User: %s' % oldIsOxUser)

	if not oldIsOxUser and not newIsOxUser:
		# Neither old nor new is OX user. Nothing to do.
		return

	blacklisted = ['Administrator', 'join-backup', 'join-slave', 'spam']

	username = ''
	if old.get('uid'):
		username = old['uid'][0]
		ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: old username: {}'.format(username))
	if new.get('uid'):
		username2 = new['uid'][0]
		ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: new username: {}'.format(username2))
		if username2 != username:
			if username2 in blacklisted or username in blacklisted:
				ud.debug(ud.LISTENER, ud.WARN, 'ox-user: not allowed to change users in blacklist (%s)' % username)
				return
		username = username2

	groupwareoptions = []

	email = '@'
	if new:
		if new.get('mailPrimaryAddress'):
			email = new['mailPrimaryAddress'][0]
			ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: new mailPrimaryAddress: {}'.format(email))
			groupwareoptions.append("--email=%s" % email)
			groupwareoptions.append("--imaplogin=%s" % email)

			aliases = '%s' % email
			aliaseslist = [email]
			if new.get('mailAlternativeAddress'):
				mailalternativeaddress = new['mailAlternativeAddress']
				for alias in mailalternativeaddress:
					aliases = '%s,%s' % (aliases, alias)
					aliaseslist.append(alias)
				ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: aliases %s' % aliases)
			groupwareoptions.append("--aliases=%s" % aliases)

	oldemail = '@'
	if old:
		if old.get('mailPrimaryAddress'):
			oldemail = old['mailPrimaryAddress'][0]
			ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: old mailPrimaryAddress: {}'.format(oldemail))
	#
	# Context bestimmen:
	#

	domain = ''
	olddomain = ''
	contextname = ''
	contextnum = ''
	oxhomeserver = ''
	admuser = ''
	admpass = ''
	oldcontextname = ''
	oldcontextnum = ''
	oldoxhomeserver = ''
	oldadmuser = ''
	oldadmpass = ''
	if new:
		domain = email.split('@')[1]
	else:
		domain = oldemail.split('@')[1]
	olddomain = oldemail.split('@')[1]

	with AsRoot():
		try:
			ldapCon = univention.uldap.getMachineConnection(ldap_master=False)
		except IOError as ex:
			raise ListenerTools_RuntimeError(ex)

	if new:
		result_id = ldapCon.search(base='cn=domain,cn=mail,%s' % ldapbase, filter='(cn=%s)' % domain, attr=['oxContextName'])
		if result_id:
			contextname = result_id[0][1].values()[0][0]

		result_id = ldapCon.search(base='cn=open-xchange,%s' % ldapbase, filter='(cn=%s)' % contextname, attr=['oxHomeServer'])
		if result_id:
			oxhomeserver = result_id[0][1].values()[0][0]
		# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: ** oxhomeserver=%s' % oxhomeserver)
		if oxhomeserver == myfqdn:
			result_id = ldapCon.search(base='cn=open-xchange,%s' % ldapbase, filter='(cn=%s)' % contextname, attr=['oxContextIDNum'])
			if result_id:
				contextnum = result_id[0][1].values()[0][0]
				admuser, admpass = get_context_credentials(contextnum)

	if old:
		filter = '(&(cn=%s)(objectClass=oxMailDomainServerSettings))' % olddomain
		result_id = ldapCon.search(base='cn=domain,cn=mail,%s' % ldapbase, filter=filter, attr=['oxContextName'])
		if result_id:
			oldcontextname = result_id[0][1].values()[0][0]

		result_id = ldapCon.search(base='cn=open-xchange,%s' % ldapbase, filter='(cn=%s)' % oldcontextname, attr=['oxHomeServer'])
		if result_id:
			oldoxhomeserver = result_id[0][1].values()[0][0]

		if oldoxhomeserver == myfqdn:
			result_id = ldapCon.search(base='cn=open-xchange,%s' % ldapbase, filter='(cn=%s)' % oldcontextname, attr=['oxContextIDNum'])
			if result_id:
				oldcontextnum = result_id[0][1].values()[0][0]
				oldadmuser, oldadmpass = get_context_credentials(oldcontextnum)

	# Entscheidung:
	#    old new ##
	# 1)  -   1  ## E  kein altes Objekt, lokaler Kontext: new erzeugen
	# 2)  a   1  ## E  altes Objekt nicht lokal, neues Objekt lokal: new erzeugen
	# 3)  1   -  ## L  kein neues Objekt, lokaler Kontext: old loeschen
	# 4)  1   a  ## L  altes Objekt in lokalem Kontext, neues Objekt nicht in lokalem Kontext: old loeschen
	# 5)  1   2  ## LE alter und neuer Kontext lokal: old loeschen, new erzeugen
	# 6)  1   1  ## C  altes + neues Objekt, beide lokaler Kontext: change auf new
	# 7)  -   a  ## -  kein altes Objekt, nicht lokaler Kontext: nichts
	# 8)  a   -  ## -  altes Objekt geloescht, nicht lokaler Kontext: nichts
	# 9)  a   b  ## -  alter und neuer Kontext nicht lokal: nichts

	authArgs = [
		"--adminuser=%s" % admuser,
		"--adminpass=%s" % admpass,
		"--contextid=%s" % contextnum,
	]
	oldAuthArgs = [
		"--adminuser=%s" % oldadmuser,
		"--adminpass=%s" % oldadmpass,
		"--contextid=%s" % oldcontextnum,
	]

	# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: ldapobjects=%s' % result_id)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: email=%s' % email)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: domain=%s' % domain)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: contextname=%s' % contextname)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: contextnum=%s' % contextnum)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: oxhomeserver=%s' % oxhomeserver)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: oldemail=%s' % oldemail)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: olddomain=%s' % olddomain)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: oldcontextname=%s' % oldcontextname)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: oldcontextnum=%s' % oldcontextnum)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: oldoxhomeserver=%s' % oldoxhomeserver)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: myfqdn=%s' % myfqdn)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: admuser=%s' % admuser)
	ud.debug(ud.LISTENER, ud.INFO, 'ox-user: oldadmuser=%s' % oldadmuser)

	# old lokal? (3) (4) (5) (6)
	if myfqdn == oldoxhomeserver:
		ud.debug(ud.LISTENER, ud.INFO, 'ox-user: old local (3,4,5,6)')
		if contextnum != oldcontextnum or not newIsOxUser:
			# loeschen (3) (4) (5)

			oldoxuidnumber = ''
			for row in ox_list_user(oldadmuser, oldadmpass, oldcontextnum, searchpattern=username, ignorecase=True):
				if row[csv_user_row_username] == username:
					oldoxuidnumber = row[csv_user_row_uid]
			ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: Processing uid %s for old object' % oldoxuidnumber)

			for row in ox_list_group(oldadmuser, oldadmpass, oldcontextnum):
				# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: Processing row: %s' % row)
				members = row[csv_group_row_members].split(',')
				group = row[csv_group_row_groupname]
				if oldoxuidnumber in members:
					if list.__len__(members) <= 1:
						ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user({}): removing ox group: {}'.format(username, group))
						groupdelete = [
							"deletegroup",
							"--name=%s" % group,
						] + oldAuthArgs
						ox_call(groupdelete, "com.openexchange.admin.rmi.exceptions.NoSuchObjectException")

			if username == oldadmuser:
				ud.debug(ud.LISTENER, ud.INFO, 'ox-user: Not deleting admin-user %s' % username)
			else:
				if command == 'r':
					# object has moved, write ldap uidNumber into filename and oxuidnumber in this file.
					ud.debug(ud.LISTENER, ud.INFO, 'ox-user: Command = r')
					lockfilename = '/var/lock/deleteuid-%s' % old['uidNumber'][0]
					with AsRoot():
						with open(lockfilename, 'w') as lockfilehandle:
							lockfilehandle.write(oldoxuidnumber)
					ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: setting lock file for user uidNumber: %s' % old['uidNumber'][0])
				else:
					deleteuser = [
						"deleteuser",
						"--username=%s" % username,
					] + oldAuthArgs
					ud.debug(
						ud.LISTENER, ud.PROCESS,
						"ox-user: deleting ox user '{}'".format(username))
					ox_call(deleteuser, "com.openexchange.admin.rmi.exceptions.NoSuchObjectException")

	# new lokal? (1) (2) (5) (6)
	if myfqdn == oxhomeserver:
		# ldapsearch -x gidNumber=5001
		# ldapsearch -x "(&(objectClass=posixGroup)(gidNumber=5001))" cn
		# ldapsearch -x "(&(objectClass=posixGroup)(memberUid=aaa))" cn

		if new.get('gidNumber'):
			gidnumber = new['gidNumber'][0]

		filter = '(&(gidNumber=%s)(objectClass=posixGroup))' % gidnumber
		result_id = ldapCon.search(base=ldapbase, filter=filter, attr=['cn'])
		if result_id:
			prigroup = result_id[0][1].values()[0][0]
			# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: prigroup=%s' % prigroup)
		filter = '(&(memberUid=%s)(objectClass=posixGroup))' % username
		result_id = ldapCon.search(base=ldapbase, filter=filter, attr=['cn'])
		if result_id:
			othergroups = result_id
			# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: othergroups=%s' % othergroups)

		imapserver = 'localhost'
		if listener.configRegistry.get('ox/role') != 'oxae':
			filter = '(&(cn=%s)(objectClass=oxMailDomainServerSettings))' % domain
			result_id = ldapCon.search(base='cn=domain,cn=mail,%s' % ldapbase, filter=filter, attr=['oxImapServer'])
			if result_id:
				imapserver = result_id[0][1].values()[0][0]
			ud.debug(ud.LISTENER, ud.INFO, 'ox-user: imapserver=%s' % imapserver)

		groupwareoptions.append("--imapserver=%s" % imapserver)
		groupwareoptions.append("--smtpserver=localhost")

		if not listener.configRegistry.is_true("mail/dovecot"):
			for reference in new.get('univentionPolicyReference', []):
				try:
					filter = '(objectClass=univentionMailQuota)'
					result_id = ldapCon.search(base=reference, filter=filter, attr=['univentionMailQuotaMB'])
					if result_id[0][1].get('univentionMailQuotaMB', [''])[0]:
						ud.debug(ud.LISTENER, ud.INFO,
							'ox-user: Calling USER=%s /usr/sbin/univention-cyrus-set-quota' % username)
						listener.setuid(0)
						oldUSER = os.environ["USER"]
						os.environ["USER"] = username
						subprocess.call('/usr/sbin/univention-cyrus-set-quota &', shell=True)
						os.environ["USER"] = oldUSER
						listener.unsetuid()
				except:
					pass

		if new.get('oxBirthday'):
			if new['oxBirthday'][0]:
				birthday = new['oxBirthday'][0]
				text = birthday
				if '-' in text:
					birthdayiso = birthday
				else:
					day, month, year = map(int, text.split('.'))
					birthdayiso = '%s-%s-%s' % (year, month, day)
				groupwareoptions.append("--birthday=%s" % birthdayiso)

		if new.get('oxAnniversary'):
			if new['oxAnniversary'][0]:
				birthday = new['oxAnniversary'][0]
				text = birthday
				if '-' in text:
					birthdayiso = birthday
				else:
					day, month, year = map(int, text.split('.'))
					birthdayiso = '%s-%s-%s' % (year, month, day)
				groupwareoptions.append("--anniversary=%s" % birthdayiso)

		if new.get('oxNumOfChildren'):
			groupwareoptions.append("--number_of_children=%s" % new['oxNumOfChildren'][0])
		elif old and old.get('oxNumOfChildren'):
			groupwareoptions.append("--number_of_children=")

		if new.get('roomNumber'):
			groupwareoptions.append("--room_number=%s" % new['roomNumber'][0])
		elif old and old.get('roomNumber'):
			groupwareoptions.append("--room_number=")

		if new.get('employeeNumber'):
			groupwareoptions.append("--number_of_employee=%s" % new['employeeNumber'][0])
		elif old and old.get('employeeNumber'):
			groupwareoptions.append("--number_of_employee=")

		accesscombinationsuffix = ''
		if new.get('oxAccessUSM'):
			accesscombinationsuffix = '_usm'

		if new.get('oxAccess'):
			oxaccess = new['oxAccess'][0]
			if oxaccess == 'none':
				groupwareoptions.append("--mailenabled=false")
				groupwareoptions.append("--access-contacts=off")
				groupwareoptions.append("--access-webmail=off")
				groupwareoptions.append("--access-calendar=off")
				groupwareoptions.append("--access-delegate-tasks=off")
				groupwareoptions.append("--access-tasks=off")
				groupwareoptions.append("--access-edit-public-folder=off")
				groupwareoptions.append("--access-infostore=off")
				groupwareoptions.append("--access-read-create-shared-Folders=off")
				groupwareoptions.append("--access-ical=off")
				groupwareoptions.append("--access-vcard=off")
				groupwareoptions.append("--access-webdav=off")
				groupwareoptions.append("--access-webdav-xml=off")
			if oxaccess == 'webmail':
				groupwareoptions.append("--mailenabled=true")
				groupwareoptions.append("--access-combination-name=oxseforucs_webmail%s" % accesscombinationsuffix)
			if oxaccess == 'pim':
				groupwareoptions.append("--mailenabled=true")
				groupwareoptions.append("--access-combination-name=oxseforucs_pim_plus%s" % accesscombinationsuffix)
			if oxaccess == 'groupware':
				groupwareoptions.append("--mailenabled=true")
				groupwareoptions.append("--access-combination-name=oxseforucs_groupware%s" % accesscombinationsuffix)
			if oxaccess == 'premium':
				groupwareoptions.append("--mailenabled=true")
				groupwareoptions.append("--access-combination-name=oxseforucs_premium%s" % accesscombinationsuffix)
		else:
			groupwareoptions.append("--mailenabled=false")

		if new.get('oxAccessSyncMl'):
			groupwareoptions.append("--access-syncml=on")
		else:
			groupwareoptions.append("--access-syncml=off")

		if new.get('oxAccessResources'):
			groupwareoptions.append("--access-edit-resource=on")
		else:
			groupwareoptions.append("--access-edit-resource=off")

		for ldapattr, oxkey in ldap2ox.items():
			if new and new.get(ldapattr):
				groupwareoptions.append("--%s=%s" % (oxkey, new.get(ldapattr)[0]))
			elif old and old.get(ldapattr):
				groupwareoptions.append("--%s=" % oxkey)

		# if more than one value is present then set the first value into "${key}1" and the second into "${key}2"
		for ldapattr, oxmultikey in ldap2oxmulti.items():
			if new and new.get(ldapattr):
				groupwareoptions.append("--%s1=%s" % (oxmultikey, new.get(ldapattr)[0]))
				if len(new.get(ldapattr)) > 1:
					groupwareoptions.append("--%s2=%s" % (oxmultikey, new.get(ldapattr)[1]))
			elif old and old.get(ldapattr):
				groupwareoptions.append("--%s1=" % oxmultikey)
				groupwareoptions.append("--%s2=" % oxmultikey)

		groupwareoptions.append("--gui_spam_filter_capabilities_enabled=true")

		oxuidnumber = get_ox_id(admuser, admpass, contextnum, username)
		if oxuidnumber:
			ud.debug(ud.LISTENER, ud.INFO, 'ox-user: old object present: %s' % (oxuidnumber,))

		# Test for lockfile if userobject has moved.
		userhasmoved = False
		if new:
			try:
				lockfilename = '/var/lock/deleteuid-%s' % new['uidNumber'][0]
				ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: lockfilename: %s' % lockfilename)
				lockfiletext = open(lockfilename).read()
				ud.debug(ud.LISTENER, ud.PROCESS, 'ox-user: lockfiletext: %s' % lockfiletext)
				if lockfiletext:
					oxuidnumber = lockfiletext
					userhasmoved = True
					with AsRoot():
						os.remove(lockfilename)
			except:
				pass
		# newcontext != oldcontext ?
		# test, ob benutzer im context schon existiert
		if userhasmoved:
			changeuser = [
				"changeuser",
				"--userid=%s" % oxuidnumber,
				"--username=%s" % username,
			] + authArgs
			ud.debug(ud.LISTENER, ud.INFO,
				'ox-user: Modifying OpenExchange username %s, oxuid %s' % (username, oxuidnumber))
			ox_call(changeuser)

		if ((contextnum == oldcontextnum) and oxuidnumber) or userhasmoved and oldIsOxUser and newIsOxUser:
			# aendern (6)
			if new.get('mailPrimaryAddress'):
				defaultsenderaddress = new['mailPrimaryAddress'][0]
				# Aktuelle default-address auslesen

				oxdefaultsenderaddress = ''
				for row in ox_list_user(admuser, admpass, contextnum, searchpattern=username, ignorecase=True):
					if row[csv_user_row_username] == username:
						oxdefaultsenderaddress = row[csv_user_row_default_sender_address]

				if oxdefaultsenderaddress not in aliaseslist:
					groupwareoptions.append("--defaultsenderaddress=%s" % defaultsenderaddress)

			if new.get('oxLanguage'):
				language = new['oxLanguage'][0]
				oldlanguage = ''
				if old.get('oxLanguage'):
					oldlanguage = old['oxLanguage'][0]
				if language != oldlanguage:
					groupwareoptions.append("--language=%s" % language)

			if new.get('oxTimeZone'):
				timezone = new['oxTimeZone'][0]
				oldtimezone = ''
				if old.get('oxTimeZone'):
					oldtimezone = old['oxTimeZone'][0]
				if timezone != oldtimezone:
					groupwareoptions.append("--timezone=%s" % timezone)

			if email == oldemail:
				# changeuser returns error message if --email is given as argument but value dit not change
				# ===>
				# user user2 in context 10 could not be changed:
				# Server response:
				# 	 Primary mail address already exists in this context.
				newgroupwareoptions = []
				for i in groupwareoptions:
					if not i.startswith("--email="):
						newgroupwareoptions.append(i)
				groupwareoptions = newgroupwareoptions
			command = [
				"changeuser",
				"--username=%s" % username,
			] + authArgs + groupwareoptions
			ud.debug(ud.LISTENER, ud.INFO,
				'ox-user: Modifying OpenExchange settings for uid %s' % username)
			ox_call(command)
		else:
			# erzeugen (1) (2) (5)
			if newIsOxUser:

				if not oxuidnumber:
					command = ["createuser"] + authArgs
					ud.debug(ud.LISTENER, ud.PROCESS, "ox-user: Creating ox user '{}'".format(username))
				else:
					command = ["changeuser"] + authArgs
					ud.debug(ud.LISTENER, ud.INFO,
						'ox-user: Modifying OpenExchange settings for uid %s' % username)

				if new.get('mailPrimaryAddress'):
					groupwareoptions.append("--defaultsenderaddress=%s" % new['mailPrimaryAddress'][0])

				if new.get('oxLanguage'):
					groupwareoptions.append("--language=%s" % new['oxLanguage'][0])

				if new.get('oxTimeZone'):
					groupwareoptions.append("--timezone=%s" % new['oxTimeZone'][0])

				if username == admuser:
					password = admpass
				else:
					password = "dummy"

				command.append("--username=%s" % username)
				command.append("--password=%s" % password)
				command = command + groupwareoptions

				ox_call(command)
				if not oxuidnumber:
					oxuidnumber = get_ox_id(admuser, admpass, contextnum, username)
				if not oxuidnumber:
					# this should never happen
					ud.debug(ud.LISTENER, ud.ERROR, "ox-user: DB didn't return oxuidnumber for username '{}'. Probably not a OX user.\nnew: {}\nold: {}".format(username, new, old))
					raise Exception("User '{}' doesn't exist after creation.".format(username))

		filter = '(&(memberUid=%s)(objectClass=posixGroup))' % username
		result_id = ldapCon.search(base=ldapbase, filter=filter, attr=['cn'])
		if result_id:
			# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: groups found')
			oxgroups = [prigroup]
			for w in result_id:
				if w[1].values()[0][0] != prigroup:
					oxgroups.append(w[1].values()[0][0])
			ud.debug(ud.LISTENER, ud.PROCESS, "ox-user({}): oxgroups: {}".format(username, oxgroups))

			# Benutzer zu Gruppen hinzufügen, bzw. Gruppe mit Benutzer anlegen:
			if oxuidnumber:
				for group in oxgroups:
					# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: processing group %s' % (w[1].values()[0][0]))
					# Gruppe suchen:
					foundgroup = False
					for row in ox_list_group(admuser, admpass, contextnum):
						if row[csv_group_row_groupname].lower() == group.lower():
							foundgroup = True
							break
					if not foundgroup:
						ud.debug(ud.LISTENER, ud.PROCESS, "ox-user({}): creating ox group: '{}'".format(username, group))

					# Displayname zu Gruppe finden:
					# 'description'
					# 'cn'
					filter = '(&(cn=%s)(objectClass=oxGroup))' % group
					result_id = ldapCon.search(base=ldapbase, filter=filter, attr=['cn', 'description'])
					# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: result_id (group): %s(%s)' % (result_id, group))
					groupdisplayname = group
					if result_id:
						if result_id[0][1].get('description'):
							groupdisplayname = result_id[0][1]['description'][0]
							# ud.debug(ud.LISTENER, ud.INFO, 'ox-user: groupsdisplayname: %s' % groupdisplayname)
						# gruppe existiert nicht und sicherheitshalber, falls die Gruppe doch bereits exisitiert.
						command = [
							"changegroup" if foundgroup else "creategroup",
							"--name=%s" % group,
							"--displayname=%s" % groupdisplayname,
							"--addmembers=%s" % oxuidnumber,
						] + authArgs
						ud.debug(
							ud.LISTENER, ud.INFO,
							'ox-user: Handling group membership for uid %s' % username)
						ox_call(command, "Member already exists in group")
			# Benutzer aus Gruppen entfernen:
			# OX-Gruppen des User-Objektes finden, in denen der Benutzer ist (oxgroups)
			# alle anderen Gruppen auf OX suchen (oxgroupmap)
			nooxgroups = []
			for row in ox_list_group(admuser, admpass, contextnum):
				if not row[csv_group_row_groupname] in oxgroups:
					if row[csv_group_row_groupname] != 'users':
						tmpoxgroup = [
							row[csv_group_row_groupname],
							row[csv_group_row_members],
						]
						nooxgroups.append(tmpoxgroup)
			ud.debug(ud.LISTENER, ud.INFO,
				'ox-user: User in OX-groups: %s' % oxgroups)
			ud.debug(ud.LISTENER, ud.INFO,
				'ox-user: User not in OX-groups: %s' % nooxgroups)
			# in diesen Gruppen auf OX suchen, ob Benutzer drin ist:
			for grouprow in nooxgroups:
				groupuserlist = grouprow[1].split(',')
				if oxuidnumber in groupuserlist:
					# ja: Benutzer aus Gruppe entfernen

					# Prüfen, ob Benutzer der letzte der Gruppe war:
					if list.__len__(groupuserlist) <= 1:
						# ja: Gruppe löschen
						ud.debug(ud.LISTENER, ud.PROCESS, "ox-user({}): removing ox group: '{}'".format(username, grouprow[0]))
						deletegroup = [
							"deletegroup",
							"--name=%s" % grouprow[0],
						] + authArgs
						ox_call(deletegroup, "com.openexchange.admin.rmi.exceptions.NoSuchObjectException")

					else:
						ud.debug(ud.LISTENER, ud.INFO, "ox-user({}): removing ox user from group: '{}'".format(username, grouprow[0]))
						changegroup = [
							"changegroup",
							"--name=%s" % grouprow[0],
							"--displayname=%s" % groupdisplayname,
							"--removemembers=%s" % oxuidnumber,
						] + authArgs
						ox_call(changegroup)


def get_context_credentials(contextid):
	admuser = 'oxadmin-context%s' % (contextid,)
	if admuser == 'oxadmin-context10':
		admuser = 'oxadmin'
	admpass = get_credentials('/etc/ox-secrets/context%s.secret' % (contextid,))
	return admuser, admpass


def get_ox_id(admuser, admpass, contextnum, username):
	for row in ox_list_user(admuser, admpass, contextnum, searchpattern=username, ignorecase=True):
		if row[csv_user_row_username] == username:
			return row[csv_user_row_uid]


if __name__ == '__main__':
	handler.process_pending()
