# -*- coding: utf-8 -*-
#
# Univention Admin Modules
#  hook definitions
#
# Copyright (C) 2004-2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import re
import datetime
import univention.debug
import univention.admin.modules
import univention.admin.uexceptions
import univention.admin.localization
import univention.uldap

translation=univention.admin.localization.translation('univention.admin.hooks.d.ox')
_=translation.translate

class oxAccess(object):
	type='oxAccess'

	def check_syntax(self, module):
		if module['username']:
			username=module['username']
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.UserName: username: %s' % username)
			unicode_username=username.decode("utf-8")
			p1 = re.compile('(?u)(^[a-zA-Z])[a-zA-Z0-9._-]*([a-zA-Z0-9]$)')
			p2 = re.compile('(?u)^\w([\w -.]*\w)?$')
			return p1.match(unicode_username) and p2.match(unicode_username)
		return False

	def check_oxaccess(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: check_oxaccess')
		try:
			return module['isOxUser'] == 'OK'
		except:
			pass
		return False

	def getOxResources(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: getOxResources')
		return module.oldattr.get('oxResourceMailAddress')

	def check_mailaddr(self, module):
		if module['mailPrimaryAddress']:
			domain = module['mailPrimaryAddress'].rsplit('@')[-1]
			filter = '(&(objectClass=oxMailDomainServerSettings)(cn=%s))' % domain
			result = module.lo.searchDn(filter=filter)

			if not result:
				raise univention.admin.uexceptions.valueError, _("The mail address' domain does not match any mail domain object.")
			else:
				univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: ldap result: %s' % result)
		else:
			raise univention.admin.uexceptions.valueError, _("The primary mail address is required for Open-Xchange users. Currently the users' primary mail address is not set.")

	def check_displayname(self, module):
		uidSearchstring = '|'
		if module.oldinfo:
			myOldUid=module.oldinfo.get('username','')
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAcces: My OldUid: %s' % myOldUid)
			if myOldUid:
				uidSearchstring += '(uid=%s)' % myOldUid
		if module.info:
			myUid=module['username']
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAcces: My Uid: %s' % myUid)
			if myUid:
				uidSearchstring += '(uid=%s)' % myUid
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: Attribute oxDisplayName: %s' % module[ 'oxDisplayName' ])
		if module['oxDisplayName']:
			oxDisplayName=''
			oxDisplayName=module['oxDisplayName']
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: OX Display Name: %s' % oxDisplayName)
			univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: ldap searchstring: (&(objectClass=oxUserObject)(oxDisplayName=%s)(!(%s)))' % (oxDisplayName, uidSearchstring))
			filter = '(&(objectClass=oxUserObject)(oxDisplayName=%s)(!(%s)))' % (oxDisplayName, uidSearchstring)
			result = module.lo.searchDn(filter=filter)
			if result:
				raise univention.admin.uexceptions.valueError, _('OX Display name "%s" is already in use.') % oxDisplayName
		else:
			raise univention.admin.uexceptions.valueError, _('OX Display name has to be set for open-xchange users.')

	def check_date(self, datestring):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: Verfying date value: %s' % datestring)
		if '-' in datestring:
			year, month, day = map(int, datestring.split('-'))
		else:
			day, month, year = map(int, datestring.split('.'))
		date = datetime.date(year, month, day)
		iso = date.isoformat()
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: Value of oxAccess: %s' % iso)

	def check_syntax_date(self, module):
		if module['oxBirthday']:
				try:
					self.check_date(module['oxBirthday'])
				except ValueError:
					raise univention.admin.uexceptions.valueError, _('Birthday must be in format \'YYYY-MM-DD\' or \'TT-MM-JJJJ\'')
		if module['oxAnniversary']:
				try:
					self.check_date(module['oxAnniversary'])
				except ValueError:
					raise univention.admin.uexceptions.valueError, _('Anniversary must be in format \'YYYY-MM-DD\' or \'TT-MM-JJJJ\'')

	def check_firstname(self, module):
		try:
			if module['firstname']:
				return True
			else:
				return False
		except:
			return False

	def hook_open(self, module):
		global translation, _
		translation=univention.admin.localization.translation('univention.admin.handlers.oxaccess')
		_=translation.translate

		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _open called')

	def hook_ldap_pre_create(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_pre_create called')
		hasOxAccess=self.check_oxaccess(module)
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: oxacess: %s' % hasOxAccess)

		# disable OX integration if mail address has not been specified --> allow to create users via CLI
		if hasOxAccess and not module.info.get('mailPrimaryAddress'):
			module['isOxUser'] = 'Not'
			hasOxAccess = self.check_oxaccess(module)

		self.check_syntax_date(module)
		if hasOxAccess:
			#if not self.check_syntax(module):
			#	raise univention.admin.uexceptions.valueError, 'Username must only contain numbers, letters and dots!'
			self.check_displayname(module)
			if module['username'] != 'oxadmin':
				self.check_mailaddr(module)
			if not self.check_firstname(module):
				raise univention.admin.uexceptions.valueError, _('First name has to be set for open-xchange users.')

	def hook_ldap_addlist(self, module, al = []):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_addlist called')
		return al

	def hook_ldap_post_create(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_post_create called')

	def hook_ldap_pre_modify(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_pre_modify called')
		hasOxAccess=self.check_oxaccess(module)
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: oxacess: %s' % hasOxAccess)
		self.check_syntax_date(module)
		if hasOxAccess:
			#if not self.check_syntax(module):
			#	raise univention.admin.uexceptions.valueError, 'Username must only contain numbers, letters and dots!'
			self.check_displayname(module)
			self.check_mailaddr(module)
			if not self.check_firstname(module):
				raise univention.admin.uexceptions.valueError, _('First name has to be set for open-xchange users.')

	def hook_ldap_modlist(self, module, ml = []):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_modlist called')
		return ml

	def hook_ldap_post_modify(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_post_modify called')

	def hook_ldap_pre_remove(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_pre_remove called')
		resources = self.getOxResources(module)
		if resources:
			raise univention.admin.uexceptions.valueError, _('The user %s cannot be removed because he is admin of the following resources: %s') % (module.oldinfo.get('username','???'), ', '.join(resources))

	def hook_ldap_post_remove(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxAccess: _ldap_post_remove called')
