# -*- coding: utf-8 -*-
#
# Univention Admin Modules
#  hook definitions
#
# Copyright (C) 2004-2015 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import univention.debug
import univention.admin.modules
import univention.admin.uexceptions
import univention.admin.localization
import univention.uldap

translation=univention.admin.localization.translation('univention.admin.hooks.d.ox')
_=translation.translate

class oxGroupHook(object):
	type='oxGroupHook'

	def check_mailaddr(self, module):
		if module['mailAddress']:
			domain = module['mailAddress'].rsplit('@')[-1]
			filter = '(&(objectClass=oxMailDomainServerSettings)(cn=%s))' % domain
			result = module.lo.searchDn(filter=filter)

			if not result:
				raise univention.admin.uexceptions.valueError, _("The mail address' domain does not match any mail domain object.")
			else:
				univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: ldap result: %s' % result)

	def hook_open(self, module):
		global translation, _
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _open called')
		translation=univention.admin.localization.translation('univention.admin.handlers.oxaccess')
		_=translation.translate

	def hook_ldap_pre_create(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_pre_create called')
		self.check_mailaddr(module)

	def hook_ldap_addlist(self, module, al = []):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_addlist called')
		return al

	def hook_ldap_post_create(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_post_create called')

	def hook_ldap_pre_modify(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_pre_modify called')
		self.check_mailaddr(module)

	def hook_ldap_modlist(self, module, ml = []):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_modlist called')
		return ml

	def hook_ldap_post_modify(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_post_modify called')

	def hook_ldap_pre_remove(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_pre_remove called')

	def hook_ldap_post_remove(self, module):
		univention.debug.debug(univention.debug.ADMIN, univention.debug.INFO, 'admin.syntax.hook.oxGroupHook: _ldap_post_remove called')
